import 'package:flutter/material.dart';
import 'fx_calendar_header.dart';
import 'fx_calendar_item.dart';
import 'fx_calendar_manager.dart';
import 'utils/solar_util.dart';
import 'utils/solar.dart';
import 'utils/holiday_util.dart';
import 'utils/holiday.dart';
import 'utils/lunar_util.dart';
import 'utils/lunar.dart';

const double FxDateTimeHeaderHeight = 80;
class FxCalendar extends StatefulWidget {
  FxCalendar({
    Key key,
    DateTime dateTime,
    FxItemModel calendarItemModel,
    this.fxSelectedDateTime,
    this.isScrollPage = true
  }) : dateTime = dateTime ?? DateTime.now(),
       fxCalendarItemModel = calendarItemModel ?? FxItemModel(),
       super(key: key);
  
  factory FxCalendar.style1({
    Key key,
    DateTime dateTime,
    FxItemModel calendarItemModel,
    FxSelectedDateTime selectedDateTime,
    bool isScrollPage
  }) = _FxCalendarStyle1;

  factory FxCalendar.style2({
    Key key,
    DateTime dateTime,
    FxItemModel calendarItemModel,
    FxSelectedDateTime selectedDateTime,
    bool isScrollPage
  }) = _FxCalendarStyle2;
  
  // 指定初始化显示的日期
  final DateTime dateTime;
  // 是否可以左右滚动
  final bool isScrollPage;
  /// 正常显示的背景颜色
  final FxItemModel fxCalendarItemModel;
  /// 回调
  final FxSelectedDateTime fxSelectedDateTime;

  @override
  _FxCalendarState createState() => _FxCalendarState();
}

class _FxCalendarState extends State<FxCalendar> implements FxCalendarBuildModelDelegate {
  // 日期管理
  FxCalendarManager _fxCalendarManager;
  // 页面滚动控制
  PageController _pageController;
  // 当前页面的page值
  int _currentPage;
  // 滚动状态
  bool _scrollerEnd = true;

  @override
  void initState() {
    super.initState();
    _fxCalendarManager = FxCalendarManager(
      initYear: widget.dateTime.year, 
      initMonth: widget.dateTime.month, 
      fxDelegate: this
    );
  }

  @override
  void currentLayoutDateTime(DateTime dateTime) {
    if (widget.fxCalendarItemModel.isShowSolarCalendar == false) {
      return;
    }
    // 获取24节气列表
    SolarUtil solarUtil = SolarUtil();
    solarUtil.jqDictInfoPaiYue(dateTime.year - 1);
    solarUtil.jqDictInfoPaiYue(dateTime.year);
    solarUtil.jqDictInfoPaiYue(dateTime.year + 1);
  }

  @override
  FxItemModel buildCalendarModel(bool isLastNextMonth, int year, int month, int day) {
    Holiday holiday = widget.fxCalendarItemModel.isShowHoliday ? HolidayUtil.getHoliday(year, month, day) : null;
    Lunar lunar = widget.fxCalendarItemModel.isShowLunarCalendar ? LunarUtil.fromYMD(year, month, day).getLunarDay() : null;
    Solar solar = widget.fxCalendarItemModel.isShowSolarCalendar ? SolarUtil.getSolar(year, month, day) : null;
    FxItemDetailModel model = FxItemModel.more(
      normalBackColor: widget.fxCalendarItemModel.normalBackColor, 
      selectedBackColor: widget.fxCalendarItemModel.selectedBackColor, 
      normalTitleColor: widget.fxCalendarItemModel.normalTitleColor,
      selectedTitleColor: widget.fxCalendarItemModel.selectedTitleColor,
      otherMonthTitleColor: widget.fxCalendarItemModel.otherMonthTitleColor,
      isShowLastNextMonth: widget.fxCalendarItemModel.isShowLastNextMonth,
      holiday: holiday,
      lunar: lunar,
      solar: solar
    );
    model.isLastNextMonth = isLastNextMonth;
    model.year = year;
    model.month = month;
    model.day = day;
    return model;
  }

  @override
  Widget build(BuildContext context) {
    return Container(
       child: Text(
         '子类实现布局'
       ),
    );
  }
}

/// 样式1布局
class _FxCalendarStyle1 extends FxCalendar {

  _FxCalendarStyle1({
    Key key,
    DateTime dateTime,
    FxItemModel calendarItemModel,
    FxSelectedDateTime selectedDateTime,
    bool isScrollPage
  }) : super( key: key,
              dateTime: dateTime, 
              calendarItemModel: calendarItemModel, 
              fxSelectedDateTime: selectedDateTime, 
              isScrollPage: isScrollPage);
  
  @override
  _FxCalendarState createState() => __FxCalendarStyle1State();
}

class __FxCalendarStyle1State extends _FxCalendarState {
  // 选中的日期
  String _selectedYMD;

  @override
  void initState() { 
    super.initState();
    _selectedYMD = '${widget.dateTime.year} + ${widget.dateTime.month} + ${widget.dateTime.day}';
    _pageController = PageController(initialPage: 1);
    // 初始化三个月的数据
    _fxCalendarManager.updateDate(0);
  }
  
  // 日历可滚动布局，
  Widget pageContentWidget(DateTime dateTime, List<FxItemModel> modelList) {
    Column column = Column(
      children: [
        Container(
          height: FxDateTimeHeaderHeight,
          child: Padding(
              padding: EdgeInsets.only(top: 15),
              child: Text(
                '${dateTime.year} 年 ${dateTime.month} 月',
                textAlign: TextAlign.center
              ),
            ),
        ),
        Container(
          height: MediaQuery.of(context).size.width / 7 * (_fxCalendarManager.selectedMonthList.length / 7),
          child: GridView.builder(
            scrollDirection: Axis.vertical,
            itemCount: modelList.length,
            physics: NeverScrollableScrollPhysics(),
            gridDelegate: SliverGridDelegateWithFixedCrossAxisCount(
              //横轴元素个数
              crossAxisCount: 7,
              //纵轴间距
              mainAxisSpacing: 5.0,
              //横轴间距
              crossAxisSpacing: 5.0,
              //子组件宽高长度比例
              childAspectRatio: 1.0
            ),
            itemBuilder: (context, index) {
              String ymd = '${modelList[index].year} + ${modelList[index].month} + ${modelList[index].day}';
              return FxCalendarItem(
                itemModel: modelList[index],
                isSelected: ymd == _selectedYMD,
                selectedFunc: (itemModel) {
                  setState(() {
                    _selectedYMD = '${itemModel.year} + ${itemModel.month} + ${itemModel.day}';
                  });
                  if (widget.fxSelectedDateTime != null) {
                    widget.fxSelectedDateTime(DateTime(itemModel.year, itemModel.month, itemModel.day));
                  }
                }
              );
            },
          )
        )
      ],
    );
    return column;
  }
  // page 列表
  List<Widget> pageWidgetList() {
    List<Widget> widgetList = [];
    for (int i = 0; i < 3; i++) {
      List<FxItemModel> modelList = _fxCalendarManager.lastMonthList;
      DateTime dateTime = _fxCalendarManager.lastDateTime;
      if (1 == i) {
        modelList = _fxCalendarManager.selectedMonthList;
        dateTime = _fxCalendarManager.selectedDateTime;
      }else if (2 == i) {
        modelList = _fxCalendarManager.nextMonthList;
        dateTime = _fxCalendarManager.nextDateTime;
      }
      widgetList.add(pageContentWidget(dateTime, modelList));
    }
    return widgetList;
  }

  @override
  Widget build(BuildContext context) {
    return Stack(
      children: [
        Container(
          height: MediaQuery.of(context).size.width / 7 * (_fxCalendarManager.selectedMonthList.length / 7) + FxDateTimeHeaderHeight,
          child: NotificationListener(
            onNotification: (notification) {
              if (notification is ScrollStartNotification && _scrollerEnd) {
                _scrollerEnd = false;
              }else if (notification is ScrollEndNotification && !_scrollerEnd) {
                _scrollerEnd = true;
                setState(() {
                  _fxCalendarManager.updateDate(_currentPage - 1);
                });
                _pageController.jumpToPage(1);
              }else if (notification is ScrollUpdateNotification) {
                debugPrint('xxxaaaa: ${notification.dragDetails?.delta}');
                // _controller.value = 1 - notification.scrollDelta;
              }
              return false;
            },
            child: PageView(
              scrollDirection: Axis.horizontal,
              controller: _pageController,
              physics: widget.isScrollPage ? AlwaysScrollableScrollPhysics() : NeverScrollableScrollPhysics(),
              onPageChanged: (value) {
                _currentPage = value;
              },
              children: pageWidgetList(),
            ),
          ),
        ),
        FxCalendarHeader(
          dateTime: null,
          lastCallBack: () {
            setState(() {
              _fxCalendarManager.updateDate(-1);
            });
          }, nextCallBack: () {
            setState(() {
              _fxCalendarManager.updateDate(1);
            });
          }
        ),
      ],
    );
  }
}

/// 样式2布局
class _FxCalendarStyle2 extends FxCalendar {
  _FxCalendarStyle2({
    Key key,
    DateTime dateTime,
    FxItemModel calendarItemModel,
    FxSelectedDateTime selectedDateTime,
    bool isScrollPage
  }) : super( key: key,
              dateTime: dateTime, 
              calendarItemModel: calendarItemModel, 
              fxSelectedDateTime: selectedDateTime, 
              isScrollPage: isScrollPage);

  @override
  _FxCalendarState createState() => __FxCalendarStyle2State();
}

class __FxCalendarStyle2State extends _FxCalendarState {
  // 选中的开始日期
  int _selectedStartDate;
  // 选中的结束日期
  int _selectedEndDate;

  @override
  void initState() {
    super.initState();
    widget.fxCalendarItemModel.year = widget.dateTime.year;
    widget.fxCalendarItemModel.month = widget.dateTime.month;
    widget.fxCalendarItemModel.day = widget.dateTime.day;
    _selectedStartDate = widget.fxCalendarItemModel.fxDateCount;
    _pageController = PageController(initialPage: 1);
    // 初始化三个月的数据
    _fxCalendarManager.updateDate(0);
  }

  // 判断是否选中
  bool isSelectedDate(FxItemModel itemModel) {
    if (_selectedStartDate != null && _selectedEndDate != null) {
      bool isSelected = itemModel.fxDateCount >= _selectedStartDate && itemModel.fxDateCount <= _selectedEndDate;
      return isSelected;
    }
    if (_selectedStartDate != null) {
      bool isSelected = itemModel.fxDateCount == _selectedStartDate;
      return isSelected;
    }
    return false;
  }
  // 处理点击选择事件
  void dealWithSelectedDate(FxItemModel itemModel) {
    setState(() {
      if (_selectedStartDate != null && _selectedEndDate != null) {
        _selectedStartDate = itemModel.fxDateCount;
        _selectedEndDate = null;
      }else if (_selectedStartDate == null) {
        _selectedStartDate = itemModel.fxDateCount;
      }else {
        if (itemModel.fxDateCount <= _selectedStartDate) {
          _selectedStartDate = itemModel.fxDateCount;
          _selectedEndDate = null;
        }else {
          _selectedEndDate = itemModel.fxDateCount;
        }
      }
    });
  }

  List<Widget> pageWidgetList() {
    List<Widget> widgetList = [];
    for (int i = 0; i < 3; i++) {
      List<FxItemModel> modelList = _fxCalendarManager.lastMonthList;
      if (1 == i) {
        modelList = _fxCalendarManager.selectedMonthList;
      }else if (2 == i) {
        modelList = _fxCalendarManager.nextMonthList;
      }
      Container container = Container(
        height: MediaQuery.of(context).size.width / 7 * (modelList.length / 7),
        child: GridView.builder(                                                                                                                                      
          scrollDirection: Axis.vertical,
          itemCount: modelList.length,
          physics: NeverScrollableScrollPhysics(),
          gridDelegate: SliverGridDelegateWithFixedCrossAxisCount(
            //横轴元素个数
            crossAxisCount: 7,
            //纵轴间距
            mainAxisSpacing: 5.0,
            //横轴间距
            crossAxisSpacing: 5.0,
            //子组件宽高长度比例
            childAspectRatio: 1.0
          ),
          itemBuilder: (context, index) {
            return FxCalendarItem(
              itemModel: modelList[index],
              isSelected: isSelectedDate(modelList[index]),
              selectedFunc: (itemModel) => dealWithSelectedDate(itemModel)
            );
          },
        )
      );
      widgetList.add(container);
    }
    return widgetList;
  }

  @override
  Widget build(BuildContext context) {
    return Column(
      children: [
        FxCalendarHeader(
          dateTime: _fxCalendarManager.selectedDateTime,
          lastCallBack: () {
            setState(() {
              _fxCalendarManager.updateDate(-1);
            });
          }, nextCallBack: () {
            setState(() {
              _fxCalendarManager.updateDate(1);
            });
          }
        ),
        SizedBox(height: 10),
        Container(
          height: MediaQuery.of(context).size.width / 7 * (_fxCalendarManager.selectedMonthList.length / 7),
          child: NotificationListener (
            onNotification: (notification) {
              if (notification is ScrollStartNotification && _scrollerEnd) {
                _scrollerEnd = false;
              }else if (notification is ScrollEndNotification && !_scrollerEnd) {
                _scrollerEnd = true;
                setState(() {
                  _fxCalendarManager.updateDate(_currentPage - 1);
                });
                _pageController.jumpToPage(1);
              }
              return false;
            },
            child: PageView(
              scrollDirection: Axis.horizontal,
              physics: widget.isScrollPage ? AlwaysScrollableScrollPhysics() : NeverScrollableScrollPhysics(),
              controller: _pageController,
              onPageChanged: (value) {
                debugPrint('page1: $value');
                _currentPage = value;
              },
              children: pageWidgetList(),
            ),
          ),
        ),
      ],
    );
  }
}