import 'package:flutter/material.dart';
import 'fx_calendar_manager.dart';

typedef FxCalendarItemSelectedFunc = void Function(FxItemModel itemModel);
class FxCalendarItem extends StatelessWidget {

  FxCalendarItem({
    Key key,
    @required this.itemModel,
    @required this.selectedFunc,
    this.isSelected
  }) : super(key: key);

  // 当前日期对象
  final FxItemModel itemModel;
  // 选择日期回调
  final FxCalendarItemSelectedFunc selectedFunc;
  // 是否选中
  final bool isSelected;

  // 文字显示的颜色
  Color showTitleColor() {
    if (itemModel.isLastNextMonth) {
      return itemModel.otherMonthTitleColor;
    }
    if (isSelected) {
      return itemModel.selectedTitleColor;
    }
    return itemModel.normalTitleColor;
  }
  // 背景颜色
  Color showBackColor() {
    if (itemModel.isLastNextMonth) {
      return itemModel.normalBackColor;
    }
    if (isSelected) {
      return itemModel.selectedBackColor;
    }
    return itemModel.normalBackColor;
  }
  // 是否隐藏
  bool isHidden() {
    if (itemModel.isShowLastNextMonth) {
      return false;
    }
    return itemModel.isLastNextMonth;
  }
  // 角标内容
  String markerText() {
    if (itemModel is FxHolidayDelegate) {
      String text = (itemModel as FxHolidayDelegate).holidayName;
      return text;
    }
    return '';
  }
  // subTitle
  String subText() {
    String text = '';
    if (itemModel is FxSolarDelegate) {
      text = (itemModel as FxSolarDelegate).solarName;
    }
    if (text.length == 0 && (itemModel is FxLunarDelegate)) {
      text = (itemModel as FxLunarDelegate).lunarName;
    }
    return text;
  }

  @override
  Widget build(BuildContext context) {
    return Stack(
      children: [
        Positioned.fill(
          child: FlatButton(
            padding: EdgeInsets.zero,
            child: ClipOval(
              child: Opacity(
                opacity: isHidden() ? 0 : 1,
                child: Transform.translate(
                  offset: Offset(0, -12),
                  child: Text(
                    '\n${itemModel.day}',
                    style: TextStyle(
                      color: showTitleColor(),
                    ),
                  ),
                )
              )
            ),
            color: showBackColor(),
            onPressed: () {
              if (itemModel.isLastNextMonth) {
                return;
              }
              selectedFunc(itemModel);
            },
          ),
        ),
        Align(
          alignment: Alignment(0.9, -1.0),
          child: Opacity(
            opacity: isHidden() ? 0 : 1,
            child: Text(
              markerText(),
              style: TextStyle(
                color: Colors.redAccent,
                fontSize: 14
              ),
            ),
          )
        ),
        Align(
          alignment: Alignment.bottomCenter,
          child: Opacity(
            opacity: isHidden() ? 0 : 1,
            child: Text(
              subText(),
              style: TextStyle(
                color: Colors.blueGrey[100],
                fontSize: 12
              ),
            ),
          )
        )
      ],
    );
  }
}