import 'lunar.dart';

void main(List<String> args) {
  LunarUtil lunarUtil = LunarUtil(DateTime.now());
  print(lunarUtil.getLunarDay());
}
/// 计算农历时间
class LunarUtil {

  LunarUtil(
    this.dateTime
  ) {
    _convert();
  }

  LunarUtil.fromYMD(
    int year,
    int month,
    int day
  ): this(DateTime(year, month, day));

  final DateTime dateTime;

  List<int> result;

  static List<int> lunarInfo = [ 0x04bd8, 0x04ae0, 0x0a570, 0x054d5,  
            0x0d260, 0x0d950, 0x16554, 0x056a0, 0x09ad0, 0x055d2, 0x04ae0,  
            0x0a5b6, 0x0a4d0, 0x0d250, 0x1d255, 0x0b540, 0x0d6a0, 0x0ada2,  
            0x095b0, 0x14977, 0x04970, 0x0a4b0, 0x0b4b5, 0x06a50, 0x06d40,  
            0x1ab54, 0x02b60, 0x09570, 0x052f2, 0x04970, 0x06566, 0x0d4a0,  
            0x0ea50, 0x06e95, 0x05ad0, 0x02b60, 0x186e3, 0x092e0, 0x1c8d7,  
            0x0c950, 0x0d4a0, 0x1d8a6, 0x0b550, 0x056a0, 0x1a5b4, 0x025d0,  
            0x092d0, 0x0d2b2, 0x0a950, 0x0b557, 0x06ca0, 0x0b550, 0x15355,  
            0x04da0, 0x0a5d0, 0x14573, 0x052d0, 0x0a9a8, 0x0e950, 0x06aa0,  
            0x0aea6, 0x0ab50, 0x04b60, 0x0aae4, 0x0a570, 0x05260, 0x0f263,  
            0x0d950, 0x05b57, 0x056a0, 0x096d0, 0x04dd5, 0x04ad0, 0x0a4d0,  
            0x0d4d4, 0x0d250, 0x0d558, 0x0b540, 0x0b5a0, 0x195a6, 0x095b0,  
            0x049b0, 0x0a974, 0x0a4b0, 0x0b27a, 0x06a50, 0x06d40, 0x0af46,  
            0x0ab60, 0x09570, 0x04af5, 0x04970, 0x064b0, 0x074a3, 0x0ea50,  
            0x06b58, 0x055c0, 0x0ab60, 0x096d5, 0x092e0, 0x0c960, 0x0d954,  
            0x0d4a0, 0x0da50, 0x07552, 0x056a0, 0x0abb7, 0x025d0, 0x092d0,  
            0x0cab5, 0x0a950, 0x0b4a0, 0x0baa4, 0x0ad50, 0x055d9, 0x04ba0,  
            0x0a5b0, 0x15176, 0x052b0, 0x0a930, 0x07954, 0x06aa0, 0x0ad50,  
            0x05b52, 0x04b60, 0x0a6e6, 0x0a4e0, 0x0d260, 0x0ea65, 0x0d530,  
            0x05aa0, 0x076a3, 0x096d0, 0x04bd7, 0x04ad0, 0x0a4d0, 0x1d0b6,  
            0x0d250, 0x0d520, 0x0dd45, 0x0b5a0, 0x056d0, 0x055b2, 0x049b0,  
            0x0a577, 0x0a4b0, 0x0aa50, 0x1b255, 0x06d20, 0x0ada0 ];
  static List<String> nStr1 = [ "日", "一", "二", "三", "四", "五", "六", "七",  
            "八", "九", "十" ];
  static List<String> nStr2 = [ "初", "十", "廿", "卅", "　" ];  
  static List<String> monthNong = [ "正", "正", "二", "三", "四", "五", "六",  
            "七", "八", "九", "十", "冬", "腊" ];  

  /// 将传入的日期转换为农历 
  void _convert() {
    // 基准时间 1900-01-31是农历1900年正月初一
    DateTime baseDateTime = DateTime(1900, 1, 31);
    // 偏移量（天）  
    int offset = dateTime.difference(baseDateTime).inDays;
    // 基准时间在天干地支纪年法中的位置  
    int monCyl = 14; // 1898-10-01是农历甲子月  
    int dayCyl = offset + 40; // 1899-12-21是农历1899年腊月甲子日  
    // 得到年数  
    int i;  
    int temp = 0;  
    for (i = 1900; i < 2050 && offset > 0; i++) {  
      temp = _totalDaysOfYear(i); // 农历每年天数  
      offset -= temp;  
      monCyl += 12;  
    }  
    if (offset < 0) {  
      offset += temp;  
      i--;  
      monCyl -= 12;  
    }
    int year = i; // 农历年份  
    int yearCyl = i - 1864; // 1864年是甲子年  

    int leap = _leapMonth(i); // 闰哪个月  
    bool isLeap = false;
    int j;  
    for (j = 1; j < 13 && offset > 0; j++) {  
      // 闰月  
      if (leap > 0 && j == (leap + 1) && isLeap == false) {
        --j;  
        isLeap = true;  
        temp = _leapDays(year);  
      } else {  
        temp = _monthDays(year, j);  
      }  
      // 解除闰月  
      if (isLeap == true && j == (leap + 1)) {
        isLeap = false;
      }
      offset -= temp;
      if (isLeap == false) {
        monCyl++;
      }
    }  
    if (offset == 0 && leap > 0 && j == leap + 1)
      if (isLeap) {
        isLeap = false;
      } else {  
        isLeap = true;
        --j;
        --monCyl;
      }  
    if (offset < 0) {  
      offset += temp;
      --j;
      --monCyl;
    }
    int month = j;        // 农历月份  
    int day = offset + 1; // 农历天
    result = [year, month, day, isLeap ? 1 : 0, yearCyl, monCyl, dayCyl];  
  }

  /// 输出格式：[五月, 十九]
  Lunar getLunarDay() {
    String ymd = '${dateTime.year}' + '-' + '${dateTime.month}' + '-' + '${dateTime.day}';
    String mm = (result[3] == 1 ? "闰" : "") + monthNong[result[1]] + '月';
    String dd = _chineseDay(result[2]);
    Lunar lunar = Lunar.init(date: ymd, month: mm, day: dd);
    return lunar;
  }

  /// 农历年份的总天数 
  /// [year] 农历年份
  static int _totalDaysOfYear(int year) {  
    int sum = 348; // 29*12  
    for (int i = 0x8000; i > 0x8; i >>= 1) {  
        sum += (lunarInfo[year - 1900] & i) == 0 ? 0 : 1; // 大月+1天  
    }  
    return (sum + _leapDays(year)); // +闰月的天数  
  }

  /// 农历 year年闰月的天数 
  /// [year] 农历年份
  static int _leapDays(int year) {  
    int result = 0;  
    if (_leapMonth(year) != 0) {  
        result = (lunarInfo[year - 1900] & 0x10000) == 0 ? 29 : 30;  
    }  
    return result;  
  }

  /// 传农历 year年闰哪个月 1-12 , 没闰传回 0 
  /// [year] 农历年份
  static int _leapMonth(int year) {  
    return (lunarInfo[year - 1900] & 0xf);  
  }

  /// 农历 y年m月的总天数
  /// [y] 农历年份
  /// [m] 农历月份
  static int _monthDays(int y, int m) {  
    return ((lunarInfo[y - 1900] & (0x10000 >> m)) == 0 ? 29 : 30);  
  }

  /// 中文日期 
  /// [day] 天
  static String _chineseDay(int day) {  
    String result;  
    switch (day) {  
      case 10:  
        result = "初十";  
        break;  
      case 20:  
        result = "二十";  
        break;  
      case 30:  
        result = "三十";  
        break;  
      default:  
        result = nStr2[day ~/ 10];// 取商  
        result += nStr1[day % 10];// 取余  
    }  
    return (result);  
  }
}